from pyAlice.messages.errors_messages import EMBEDDED_ERRORS_MESSAGE


class BaseErrors(Exception):
    def __init__(self, text, context=None, language="en", *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.msg = text
        self.context = context
        self.language = language

    def get_dialog(self, key):
        return EMBEDDED_ERRORS_MESSAGE.get(f"{key}-{self.language}")

    def __str__(self):
        return f"Base Error: {self.get_dialog(self.msg)}"


class SettingsErrors(BaseErrors):
    def __str__(self):
        return f"Settings Error: {self.get_dialog(self.msg).format(self.context)}"
    
class KeyWordsErrors(BaseErrors):
    def __str__(self):
        return f"Key words Error: {self.get_dialog(self.msg).format(self.context)}"
    
class MessageErrors(BaseErrors):
    def __str__(self):
        return f"Message Error: {self.get_dialog(self.msg).format(self.context)}"
    
class StorageErrors(BaseErrors):
    def __str__(self):
        return f"Storage Error: {self.get_dialog(self.msg).format(self.context)}"


from pyAlice.errors.errors import KeyWordsErrors
from pyAlice.base import Base


class KeyWord(Base):
    def __init__(self, text, settings, start_time):
        self.start_time = start_time
        self.settings = settings
        self.key_words = settings.KEY_WORD
        self.text = text
        self.add_log("key_word_init_log", color="green", start_time=self.start_time)

    def key_word(self):
        answer = []
        answer_buttons = []
        answer_messages = []
        answer_events = []
        for key, value in self.key_words.items():
            for item in value["key_words"]:
                if item in self.text:
                    answer.append(key)
                    answer_buttons.extend(value["buttons"])
                    answer_messages.append(value["message"])
                    answer_events.append(value["event"])
        return {
            "key_words": answer,
            "events": answer_events,
            "messages": answer_messages,
            "buttons": answer_buttons,
        }








if __name__ == "__main__":


    a = KeyWord("../testing/key_word", "привет тебе нужна помощь функции")

    print(a.key_word())


import os
from sys import platform

from pyAlice.functions import clear_console


class Simulation:
    def __init__(self, handler):
        clear_console()
        self.handler = handler
        self.user_data = ""
        self.response()

    def request(self):
        self.user_data = input()
        return self.handler({
            "meta": {
                "locale": "ru-RU",
                "timezone": "UTC",
                "client_id": "ru.yandex.searchplugin/7.16 (none none; android 4.4.2)",
                "interfaces": {
                    "screen": {},
                    "payments": {},
                    "account_linking": {}
                }
            },
            "session": {
                "message_id": 5,
                "session_id": "5175cbbd-e484-4858-b253-f4c7b4632385",
                "skill_id": "37ccbb22-0b07-4a93-85c0-8517d17ccda7",
                "user": {
                    "user_id": "150725BC0540B5EDE3E831A6DB9EFA4F5A3E9BD8BD7F800301393F537B2D40E6"
                },
                "application": {
                    "application_id": "EA141820AAB4284BCD741A1D9037A49D06166D2141AAE685F9360C6DEF2853B7"
                },
                "new": False,
                "user_id": "EA141820AAB4284BCD741A1D9037A49D06166D2141AAE685F9360C6DEF2853B7"
            },
            "request": {
                "command": f"{self.user_data}",
                "original_utterance": f"{self.user_data}",
                "nlu": {
                    "tokens": [item for item in self.user_data.split(" ")],
                    "entities": [],
                    "intents": {}
                },
                "markup": {
                    "dangerous_context": False
                },
                "type": "SimpleUtterance"
            },
            "version": "1.0"
        })

    def response(self):
        while True:
            req = self.request()
            print(req["response"]["text"])
            if req["response"]["end_session"] == True:
                exit()



"""
PyAlice
v0.1
"""



import datetime
import json
import os
from pathlib import Path


from pyAlice.base import Base
from pyAlice.key_word import KeyWord
from pyAlice.errors.errors import SettingsErrors, KeyWordsErrors
from pyAlice.functions import new_settings_is_valid, clear_console, pass_fun, finish_PyAlice


class PyAlice(Base):
    def __init__(self, params_alice, settings, time_now=datetime.datetime.now(), \
                 distributor=finish_PyAlice, emptiness_actions=pass_fun, successful_fun=pass_fun):
        clear_console()
        self.params_alice = params_alice if type(params_alice) is dict else json.loads(params_alice)
        self.time_now = time_now
        self.start_time = datetime.datetime.now()

        settings_is_valid = new_settings_is_valid(settings)
        if settings_is_valid != "success":
            raise SettingsErrors(settings_is_valid, language=settings.DEBUG_LANGUAGE)
        self.settings = settings
        self.add_log("configuration_options_log", color="yellow", start_time=self.start_time)

        self.end_session = False
        self.inaction = True

        self.__processing_params()
        print(Path(__file__).resolve().parents)
        return distributor(self, emptiness_actions, successful_fun)

    def init_key_word(self):
        try:
            text_for_key_words = self.params_alice["request"][self.settings.TEXT_FOR_KEY_WORDS]
            key_words = KeyWord(text=text_for_key_words, settings=self.settings, start_time=self.start_time).key_word()

            if key_words["key_words"] == []:
                self.add_log("key_word_not_found_log", color="green", start_time=self.start_time)
            else:
                self.key_word = key_words["key_words"]
                for item in key_words["messages"]:
                    if item != "$NULL":
                        if self.result_message != "":
                            old_message = self.result_message
                            # self.result_message = self.get_message(item)
                            self.add_message(item)
                            self.add_event({
                                "event": "overwritingDefaultTextEvent",
                                "old_message": old_message,
                                "new_message": self.result_message
                                }, "KeyWord")
                        else:
                            # self.result_message = self.get_message(item)
                            self.add_message(item)
                for item in key_words["events"]:
                    self.add_event({"event":item}, "KeyWord")
                # self.buttons.extend(key_words["buttons"])
                self.add_buttons(key_words["buttons"])
                self.add_log("key_word_log", color="green", start_time=self.start_time)
        except KeyWordsErrors as exc:
            raise


    def __processing_params(self):
        self.text = self.params_alice["request"][self.settings.SOURCE_TEXT]
        self.new = self.params_alice["session"]["new"]

        self.intents = self.params_alice["request"]["nlu"]["intents"]

        if self.params_alice.get("state") is not None:
            self.storage = self.params_alice["state"]["session"]
            self.add_log("storage_fill", color="green", start_time=self.start_time)
            self.add_event({
                            "event": "storageFillEvent",
                            "storage": self.storage
                            }, "__init__")
        else:
            self.add_log("storage_not_fill", color="yellow", start_time=self.start_time)
        
        if self.new:
            # self.result_message = self.settings.ALL_MESSAGES[self.settings.STARTING_MESSAGE]
            self.add_message(self.settings.STARTING_MESSAGE)
            self.add_buttons(self.settings.STARTING_BUTTONS)
            self.add_log("first_starting_end", color="green", start_time=self.start_time)
            return
        
        self.init_key_word()
        self.alice_buttons = self.get_buttons()

    def get_params_for_alice(self, type="json"):
        params_for_alice = {
            "state": {
                "session": self.storage
            },
            "session_state": self.storage,
            'version': self.params_alice['version'],
            'session': self.params_alice['session'],
            'response': {
                "buttons": self.alice_buttons,
                # Respond with the original request or welcome the user if this is the beginning of the dialog and the request has not yet been made.
                'text': self.result_message,
                # Don't finish the session after this response.
                'end_session': self.end_session
            },
        }
        if self.more_data_message != {}:
            for key, value in self.more_data_message.items():
                params_for_alice["response"][key] = value
        if type == "json":
            return json.dumps(params_for_alice)
        elif type == "dict":
            return params_for_alice
    
    def end_session_alice(self):
        self.end_session = True
        self.add_log("session_end", color="purple", start_time=self.start_time)


    




#


import os
from sys import platform

def new_settings_is_valid(new_settings):
    if type(new_settings.EVENTS) != bool:
        return "events_setting_error"
    if type(new_settings.DEBUG) != bool:
        return "debug_setting_error"
    # if type(new_settings.BUTTONS_MESSAGE) != bool:
    #     return "buttons_dialogs_auto_setting_error"
    # if (new_settings.TIME_ZONE == "" or len(new_settings.TIME_ZONE) > 5):
    #     return "time_zone_setting_error"
    # слелать проверку now
    if new_settings.TEXT_FOR_KEY_WORDS != "command" and new_settings.TEXT_FOR_KEY_WORDS != "original_utterance":
        return "text_for_key_word_setting_error"
    if len(new_settings.LANGUAGE) < 2:
        return "language_setting_error"
    if new_settings.STARTING_MESSAGE is None:
        return "starting_dialog_setting_error"
    if new_settings.SOURCE_TEXT is None and (new_settings.SOURCE_TEXT != "commands" or new_settings.SOURCE_TEXT != "original_utterance"):
        return "source_text_setting_error"
    return "success"


def end_alice_event(event, cls):
    cls.end_session_alice()

def clear_console():
    if platform == "linux" or platform == "linux2" or platform == "darwin":
        os.system("clear")
    elif platform == "win32":
        os.system("cls") 

def finish_PyAlice(cls, emptiness_actions, successful_fun):
    if cls.is_empty_events() and cls.buttons == [] and cls.key_word == []:
        return emptiness_actions(cls)
    else:
        cls.inaction = False
        return successful_fun(cls)


def pass_fun(*args, **kwargs):
    pass


embedded_message = {
  "configuration_options_log-ru": "Настройки успешно были проверены и записаны",
  "configuration_options_not_change_log-ru": "Новых настроек не постуало, поэтому класс работает на настройках по умолчанию",

  "storage_fill-ru": "В хранилище из запроса были данные и оно было скопировано",
  "storage_not_fill-ru": "В хранилище из запроса небыло данных",

  # Логи для KeyWord
  "key_word_init_log-ru": "Класс по поиску ключевых слов был проинициолизирован",
  "key_word_log-ru": "Поиск по ключевым словам прошол успешно",
  "key_word_not_found_log-ru": "Поиск по ключевым словам ничего не нашол",
#   "key_word_log_error-ru": "Поиск по ключевым словам не прошoл возникла ошибка",


  "buttons_success_founded-ru": "Все кнопки успешно найдены",




#   "get_events_log-ru": "GET: Запрос на получение 'events'",
#   "get_buttons_log-ru": "GET: Запрос на получение 'buttons'",
#   "get_key_words_log-ru": "GET: Запрос на получение 'key_words'",
#   "get_intents_log-ru": "GET: Запрос на получение 'intents'",
#   "get_storage_log-ru": "GET: Запрос на получение 'storage'",
#   "add_storage_log-ru": "Запрос на добавление в 'storage'",
#   "update_storage_log-ru": "Запрос на обнавление элемента 'storage'",
#   "update_in_add_storage_log-ru": "Запрос на обнавление элемента 'storage' из метада 'add_storage()'",


  "event_success_add-ru": "Event '{}' успешно записан",
  "event_success_call-ru": "Функция event(а) '{}' успешно выполнена",


  "session_end-ru": "Сесия переведена с статус 'False'", 

#   "not_text_log": "Текста в ответе от Алисы нет, поэтому работать с текстом нецелесообразно",

#   "configuration_options_log_error": "Настройки по какой то причине не были записаны",


#   "change_debug": "Чтобы увидеть логи поменяйте настройку 'debug' на true и предайте новые настройк классу PyAlice",

    "first_starting_end-ru": "Первый запуск успешно завершён",


    # english language
    "configuration_options_log-en": "The settings were successfully checked and recorded",

    "first_starting_end-en": "The first launch was successfully completed",
}


import datetime, json

from pyAlice.errors.errors import MessageErrors, SettingsErrors, StorageErrors
from pyAlice.messages.embedded_message import embedded_message


class Base:
    # bool
    new = False
    # str
    result_message, text = "", ""
    # dict
    intents, message, storage, events, logs, response, more_data_message = {}, {}, {}, {}, {}, {}, {}
    # list
    buttons, alice_buttons, key_word = [], [], []

    EMBEDDED_MESSAGE = embedded_message


    def add_log(self, log, color=None, context="", start_time=datetime.datetime.now()):
        if not self.settings.DEBUG:
            return
        delta_time = datetime.datetime.now() - start_time

        # time = str(datetime.datetime.now() - start_time)[5:]
        time = f"{delta_time.seconds}s {delta_time.microseconds} micros"
        log_text = self.EMBEDDED_MESSAGE.get(f"{log}-{self.settings.DEBUG_LANGUAGE}").format(context)
        self.logs[time] = f"{log_text}"
        if self.settings.LOG_OUTPUT_IMMEDIATELY:
            colors = {
                "black": "30",
                "red": "31",
                "green": "32",
                "yellow": "33",
                "blue": "34",
                "purple": "35",
                "turquoise": "36",
                "white": "37"
            }
            if color is not None:
                color_str = f"\033[{colors.get(color)}m"
                print(f"{color_str}{time}  -->  {log_text}\033[0m")
            else:
                print(f"{time}  -->  {log_text}")
    def get_message(self, message):
        answer = self.settings.ALL_MESSAGES.get(message)
        if answer is None:
            raise MessageErrors("dialog_not_found_error", context=message, language=self.settings.DEBUG_LANGUAGE)
        return answer
    
    def get_buttons_name(self):
        answer = []
        for item in self.buttons:
            if "$" in item:
                grop_buttons = self.settings.BUTTONS_GROUPS.get(item)
                if grop_buttons is None:
                    raise SettingsErrors("group_buttons_not_found_error", context=item, language=self.settings.DEBUG_LANGUAGE)
                for it in grop_buttons:
                    answer.append(it)
            else:
                answer.append(item)
        return answer

    def get_buttons(self):
        answer = []
        for item in self.get_buttons_name():
            button = self.settings.BUTTONS.get(item)
            if button is None:
                raise SettingsErrors("button_not_found_error", context=item, language=self.settings.DEBUG_LANGUAGE)
            if button not in answer:
                answer.append(button)
        self.add_log("buttons_success_founded", color="yellow", start_time=self.start_time)
        return answer
    
    def add_button(self, name):
        if name not in self.buttons:
            self.buttons.append(name)
            self.alice_buttons = self.get_buttons()
    
    def add_buttons(self, buttons):
        for item in buttons:
            self.add_button(item)
    
    def get_storage(self):
        return self.storage
    
    def get_storage_by_key(self, key):
        data = self.storage.get(key)
        if data is None:
            raise StorageErrors("item_in_storage_not_found_error", context=key, language=self.settings.DEBUG_LANGUAGE)
        return data
    
    def set_storage(self, key, data, overwrite=True):
        if key in self.storage.keys() and not overwrite:
            raise StorageErrors("overwrite_storage_error", context=key, language=self.settings.DEBUG_LANGUAGE)
        self.storage[key] = data
    
    def update_storage_by_key(self, key, data):
        if key not in self.storage.keys():
            raise StorageErrors("item_in_storage_not_found_error", context=key, language=self.settings.DEBUG_LANGUAGE)
        self.storage[key] = data
    
    def add_event(self, event, where):
        event_name = event["event"]
        if event_name != "$NULL":
            event_data = {
                "where": where,
                "when": str(datetime.datetime.now() - self.start_time)[5:],
                # "reason": self.__get_reasons("from_key_words_event_resons")
            }
            self.events[event_name] = event_data
            self.add_log("event_success_add", color="blue", context=event_name, start_time=self.start_time)

            fun = self.settings.EVENTS_LIST.get(event_name)
            if fun is not None:
                event["event_data"] = event_data
                fun(event, self)
                self.add_log("event_success_call", color="blue", context=event_name, start_time=self.start_time)
    
    def is_empty_events(self):
        if self.events == {}:
            return True
        for event in self.events.keys():
            if event not in self.settings.IGNORE_EVENTS_LIST:
                return False
        return True
    
    def add_message(self, message_name):
        self.result_message = self.get_message(message_name)
        more_data = self.settings.MORE_DATA_MESSAGES.get(message_name)
        if more_data is not None:
            self.more_data_message = json.loads(json.dumps(more_data).replace("$MESSAGE", self.result_message))
        else:
            self.more_data_message = {}


EMBEDDED_ERRORS_MESSAGE = {
  "color_logs_error-ru": "метод add_log() принимает либо color либо type",
  
  "key_word_duplicate_error-ru": "Поиск по ключевым словам вернул болле одного ключевого слова, проверьте чтобы ключевые слова не пересикались между собой",





  "events_setting_error-ru": "тип даных 'EVENTS' должен быть bool",
  "debug_setting_error-ru": "тип даных 'DEBUG' должен быть bool",
  "buttons_dialogs_auto_setting_error-ru": "тип даных 'BUTTONS_MESSAGE' должен быть bool",
  "time_zone_setting_error-ru": "'TIME_ZONE' должен выгледеть так 'UTC+3' или '+3'",
  "text_for_key_word_setting_error-ru": "'TEXT_FOR_INTENTS' и 'TEXT_FOR_KEY_WORDS' должен быть равны 'commands' или 'original_utterance'",
  "language_setting_error-ru": "Язык указан невено",
  "starting_dialog_setting_error-ru": "Стартовая фраза не указана",
  "source_text_setting_error-ru": "'SOURCE_TEXT' должен быть равным 'commands' или 'original_utterance'",






  "dialog_not_found_error-ru": "Диалог с ключом '{}' не найден",

  "group_buttons_not_found_error-ru": "Группа кнопок '{}' не найдена",
  "button_not_found_error-ru": "Кнопока '{}' не найдена",

  "item_in_storage_not_found_error-ru": "Запись в хранилище с именем '{}' не найдена",
  "overwrite_storage_error-ru": "'{}' такая запись уже сущетвует в хранилище и её перезапись невозможа, так как параметр 'overwrite' в состоянии True",

#   "get_params_type_error-ru": "Атрибут type в методы get_params должен быть равен либо 'json' либо 'dict'",


    "events_setting_error-en": "the 'EVENTS' data type should be bool",
}


import setuptools
with open(r'/Users/admin/Desktop/pyAlice/README.md', 'r', encoding='utf-8') as fh:
	long_description = fh.read()

setuptools.setup(
	name='pyaliceya',
	version='0.1.1',
	author='oboroksergey',
	author_email='oborok05@bk.ru',
	description='lib for yandex alice demo',
	long_description=long_description,
	long_description_content_type='text/markdown',
	url='https://github.com/sergey200519/pyAlice',
	packages=['pyAlice', 'pyAlice/errors', 'pyAlice/messages'],
	include_package_data=True,
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	python_requires='>=3.6',
)